@@ -16,7 +16,8 @@ B<Text::NWrap> - a simple text wrapping module
 B<Text::NWrap> is a simple interface to B<Text::Format>.  If you want
 more functionality just use B<Text::Format>.
 B<Text::NWrap> is meant to replace B<Text::Wrap>.  Its interface is
-basically identical to B<Text::Wrap>'s.
+basically identical to B<Text::Wrap>'s.  However, nothing is exported by
+default and it does not call die under any circumstance.
 
 =over 4
 
@@ -36,7 +37,7 @@ Lets you set the width of the paragraph.  Default is 72 characters wide.
     use Text::NWrap;
 
     print NWrap::wrap('','',"hello world this is some silly example",
-            " and some more text");
+            "and some more text that should explain the use");
 
     use Text::NWrap qw(wrap $columns);
 
